#ifndef _AFTER_H
#define _AFTER_H

#if defined (__uxp__)
#pragma global noalias
#pragma global novrec
#endif

/* =============================================== */
/* These include files should be standard on all   */
/* UNIX systems.                                   */
/* =============================================== */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <ctype.h>
#include <math.h>
#include <float.h>
#include <time.h>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>


#if ! defined (HAVE_MMAP)
#  if defined (__sun)
#    define HAVE_MMAP
#  endif
#endif

#if defined (HAVE_MMAP)
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h> /* mmap() is defined in this header */
#include <fcntl.h>
#endif

#ifndef _ERROR_H
#  include "error.h"
#endif
#ifndef _DMEMORY_H
#  include "dmemory.h"
#endif

#define FT_GRIB  1
#define FT_SERV  2
#define FT_CDF   3

#ifndef UCHAR
#define UCHAR unsigned char
#endif

#ifndef TRUE
#define TRUE  1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))

#define MaxLevel 1024

#define MaxCodes  277

#define S_ECHAM5  1

struct Control
{
  int     needed0;   /* var needed for process  */
  int     needed;    /* var needed for process  */
  int     selected;  /* var selected for output */
  int     detected;  /* var detected in input   */
  int     comp;      /* compute var if seleceted and not detected */
  int     sfit;
  int     hlev;
  int     plev;
  int     ivarID;
  int     ovarID;    /* 1st variable ID */
  int     ovarID2;   /* 2nd variable ID used for variance */
  int     tableID;
  int     igridID;
  int     ogridID;
  int     izaxisID;
  int     ozaxisID;
  int     nmiss0;
  int     nmiss;
  double  missval;
  double *spectral;
  double *spectral0;
  double *fourier;
  double *hybrid;
  double *hybrid0;
  double *height;
  double *grid;
  double *grid0;
  double *mean;
  double *variance;
  int    *samp;
};

struct Date
{
   int yr;
   int mo;
   int dy;
   int hr;
   int mn;
};

/* Memory Allocation */

double *DoubleAlloc(int words, char *array_name);
void   *FreeMemory(void *ptr);

/* UTIL */

void RealCopy(void *destination, void *source, int words);
void RealZero(double *field, int words);

/* FFT */

void fft_set(double *trigs, long *ifax, long n);
void fc2gp(double *trig, long *ifax, double *fc, double *gp, long nlat, long nlon, long nlev, long nfc);
void gp2fc(double *trig, long *ifax, double *gp, double *fc, long nlat, long nlon, long nlev, long nfc);

/* Convert */

/* Convert Spectral Array to new resolution */
void sp2sp(double *arrayIn, int truncIn, double *arrayOut, int truncOut);
void sp2fc(const double *sa, double *fa, const double *poli, long nlev, long nlat, long nfc, long nt);
void fc2sp(double *fa, double *sa, double *poli, int klev, int nlat, int nfc, int nt);

void Omega(double *omega, double *divergence, double *u_wind, double *v_wind,
	   double *halfpress, double *fullpress, double *dpsdx, double *dpsdy,
	   double *vct, int dimgp, int dim3gp, int nlev);

/* Physc */

void IniQuaSum(double *dest, double *src, int len);
void AddQuaSum(double *dest, double *src, int len);
void VarQuaSum(double *Variance, double *Sum, int len, int n);
void AddVector(double * restrict dest, const double *src, long len, int *nmiss, double missval);
void Add2Vectors(double *dest, double *srcA, double *srcB, int len);
void Sub2Vectors(double *dest, double *srcA, double *srcB, int len);
void MultVectorScalar(double *dest, double *src, double factor, int len, int nmiss, double missval);
void DivVectorIvector(double *dest, double *src, int *samp, int len, int *nmiss, double missval);

void windSpeed(double *speed, double *u, double *v, int dim3gp);
void dv2ps(double *div, double *pot, int lev, int trunc);
void dv2uv(double *d, double *o, double *u, double *v, double *f, double *g,
           int nt, int nsp, int nlev);
void scaluv(double *fu, double rclat[], int nlat, int lot);
void uv2dv(double *fu, double *fv, double *sd, double *sv,
           double *pol2, double *pol3, int klev, int nlat, int nt);
void geninx(long ntr, double *f, double *g);
void theta(double *pthetaf, double *pthetah, double *ph, double *ps,
           double *tf, double *ts, int levels, int dimgp, int dim3gp);
void gaussaw (double *pa, double *pw, int nlat);
void phcs(double *PNM, double *HNM, int Waves, double PMU,
          double *ZTEMP1, double *ZTEMP2);
void sh2rh(double *sphum, double *rhum, double *t, int lev,
           int dimgpout, double *level, double *fullpresshybrid);
void rh2sh(double *sphum, double *rhum, double *t, int lev,
           int dimgpout, double *level);
void MakeGeopotHeight(double *geop, double* gt, double *gq, double *ph, int nhor, int nlev);
void LayerWater (double *ww, double *ll, double pmax, double pmin,
                 int DimGP, int HalfLevels, double *vct);
void LayerCloud (double *cc, double *ll, double pmax, double pmin,
                 int DimGP, int HalfLevels, double *vct);
void Derivate(double field[], double derilam[], int levels,
              int Waves, int Latitudes, double DerivationFactor[]);

int   scanparObsolate(char *namelist, char *name, int def);
void  scancode(char *namelist, struct Control *All, int maxCodes, int *numCodes);
int   scanpar(char *namelist, char *name, int def);
int   scantime(char *namelist, int *hours, int max_hours);
void  scandarray(char *namelist, char *name, double *values, int maxValues, int *numValues);
void  lprintf(FILE *fp);

void  procstat(char *procpath, int truncation);
void  CheckAnalyses(struct Control *All);
void  CheckContent(struct Control *All, int timestep);

void  FreeSpectral(struct Control *All);
void  FreeFourier(struct Control *All);
void  FreeHybrid(struct Control *All);
void  FreeGrid(struct Control *All);
void  FreeMean(struct Control *All);
void  FreeSamp(struct Control *All);

void  AnalysisDependencies(struct Control *All, int ncodes);
void  EchamDependencies(struct Control *All, int ncodes, int type, int source);

void  printCodes(void);
void  setStartTime(void);
int   get_nfft(void);

#define    LOW_CLOUD   34
#define    MID_CLOUD   35
#define    HIH_CLOUD   36
#define    LOW_WATER   37  /* not used ?   */
#define    MID_WATER   38  /* not used ?   */
#define    HIH_WATER   39  /* not used ?   */
#define    ALL_WATER   40  /* not used ?   */

#define GEOPOTENTIAL  129
#define  TEMPERATURE  130
#define       U_WIND  131
#define       V_WIND  132
#define     HUMIDITY  133
#define           PS  134
#define        OMEGA  135
#define    VORTICITY  138
#define           TS  139
#define       STREAM  148
#define      VELOPOT  149
#define          SLP  151
#define         LNPS  152
#define   DIVERGENCE  155
#define GEOPOTHEIGHT  156
#define    RHUMIDITY  157

#define   SW_BOT_CLF  189  /* not used ?   */
#define   LW_BOT_CLF  190  /* not used ?   */
#define   SW_TOP_CLF  191  /* not used ?   */
#define   LW_TOP_CLF  192  /* not used ?   */
#define  NET_TOP_CLF  193  /* not computed */

#define    WINDSPEED  259
#define       PRECIP  260
#define      NET_TOP  261
#define      NET_BOT  262
#define     NET_HEAT  263
#define    NET_WATER  264
#define       SW_CLF  265
#define       LW_CLF  266
#define      NET_CLF  267
#define       SW_ATM  268
#define       LW_ATM  269
#define      NET_ATM  270
#define  SURF_RUNOFF  271
#define        DPSDX  273
#define        DPSDY  274
#define  FRESH_WATER  275
#define      PS_PROG  276  /* PS for prognostic timestep */
#define   HALF_PRESS  277
#define   FULL_PRESS  278
#define       THETAH  279
#define       THETAF  280


#endif /*  after.h  */
